function h = gpoly(fig)
%GPOLY Draw a polygon in a figure interactively.
%   GPOLY(FIG) draws a polygon by clicking the mouse at the
%   successive vertices of the polygon in the figure FIG.
%   A rubberband line tracks the mouse movement. 
%   H = GPOLY(FIG) Returns the handle to the polygon.
%
%   GLINE with no input arguments draws in the current figure.


if nargin<1, 
  draw_fig = gcf;
  fig = draw_fig;
  s = 'start'; 
end

if isstr(fig), 
   s = fig;
   draw_fig = gcbf;
   ud = get(draw_fig,'UserData');
   gcacolor = get(gca,'Color');
else
   s = 'start';
   draw_fig = fig; 
end

switch s
   case 'start'
   oldtag = get(draw_fig,'Tag');
   figure(draw_fig);
   ax = gca;
   if any(get(ax,'view')~=[0 90]), 
     set(draw_fig, 'Tag', oldtag);
     error('GPOLY works only for 2-D plots.');
   end
   
   % Create an invisible line to preallocate the xor line color.
   x = [0 0];
   y = [0 0];
   hpoly = line(x,y,'Visible','off','eraseMode','xor');

   bdown = get(draw_fig,'WindowButtonDownFcn');
   bup = get(draw_fig,'WindowButtonUpFcn');
   bmotion = get(draw_fig,'WindowButtonMotionFcn');
   kpress = get(draw_fig,'KeyPressFcn');
   oldud = get(draw_fig,'UserData');
   set(draw_fig,'WindowButtonDownFcn','gpoly(''down'')')
   set(draw_fig,'WindowButtonMotionFcn','gpoly(''motion'')')
   set(draw_fig,'WindowButtonUpFcn','')
   set(draw_fig,'KeyPressFcn','gpoly(''done'')')
 
   ud.hpoly = hpoly;
   ud.ptcnt=2;
   ud.pts = [];
   ud.buttonfcn = {bdown; bup; bmotion; kpress};
   ud.oldud = oldud;
   ud.oldtag = oldtag;
   set(draw_fig,'UserData',ud);
   if nargout == 1
      h = hpoly;
   end

case 'motion'
   set(draw_fig,'Pointer','crosshair');

   if isempty(ud.pts);
      return;
   end

   set(ud.hpoly,'Xdata',ud.pts(:,1),'Ydata',ud.pts(:,2),'eraseMode','xor', ...
        'linestyle','-', 'linewidth', 1.5, 'Color',1-gcacolor,'Visible','on');
   Pt2 = get(gca,'CurrentPoint'); 
   Pt2 = Pt2(1,1:2);    
   ud.pts(ud.ptcnt,:) = Pt2;

   set(draw_fig,'UserData',ud);

case 'down'   
   Pt1 = get(gca,'CurrentPoint'); 
   ud.pts = [Pt1(1,1:2); Pt1(1,1:2)];
   set(ud.hpoly,'Xdata',ud.pts(:,1),'Ydata',ud.pts(:,2),'eraseMode','xor', ...
        'linestyle','-', 'linewidth', 1.5, 'Color',1-gcacolor,'Visible','on');

   set(draw_fig,'WindowButtonDownFcn','gpoly(''down2'')','UserData',ud)
   
case 'down2'
   Pt2 = get(gca,'CurrentPoint'); Pt2 = Pt2(1,1:2);    
   ud.pts(ud.ptcnt,:) = Pt2;
   ud.ptcnt=ud.ptcnt+1;
   set(ud.hpoly,'Xdata',ud.pts(:,1),'Ydata',ud.pts(:,2),'eraseMode','normal', ...
         'linestyle','-', 'linewidth', 1.5, 'Color',1-gcacolor);
   set(draw_fig,'UserData',ud);

case 'done'
   ud.pts(ud.ptcnt,:) = ud.pts(1,:);
   set(ud.hpoly,'Xdata',ud.pts(:,1),'Ydata',ud.pts(:,2),'eraseMode','normal', ...
         'linestyle','-', 'linewidth', 1.5, 'Color',1-gcacolor);
   bfcns = ud.buttonfcn;
   set(draw_fig,'windowbuttondownfcn',bfcns{1},'windowbuttonupfcn',bfcns{2}, ...
         'windowbuttonmotionfcn',bfcns{3},'keypressfcn',bfcns{4},'Pointer','arrow', ...
		 'Tag',ud.oldtag,'UserData',ud.oldud);

otherwise
   error('Invalid call-back.');
end
